# Copyright (C) 2004-2023, Stefan Kanthak <stefan.kanthak@nexgo.de>

# * The software is provided "as is" without any warranty, neither express
#   nor implied.
# * In no event will the author be held liable for any damage(s) arising
#   from the use of the software.
# * Redistribution of the software is allowed only in unmodified form.
# * Permission is granted to use the software solely for personal private
#   and non-commercial purposes.
# * An individuals use of the software in his or her capacity or function
#   as an agent, (independent) contractor, employee, member or officer of
#   a business, corporation or organization (commercial or non-commercial)
#   does not qualify as personal private and non-commercial purpose.
# * Without written approval from the author the software must not be used
#   for a business, for commercial, corporate, governmental, military or
#   organizational purposes of any kind, or in a commercial, corporate,
#   governmental, military or organizational environment of any kind.

.SUFFIXES:

all: UACAMOLE.DLL
	RUNDLL32.EXE "$(MAKEDIR)\UACAMOLE.DLL",#1

clean:
	Erase UACAMOLE.DOS UACAMOLE.EXP UACAMOLE.LIB UACAMOLE.OBJ

clobber: clean
	Erase UACAMOLE.DLL

UACAMOLE.DLL: UACAMOLE.DOS UACAMOLE.OBJ
	LINK.EXE /LINK /ALLOWBIND:NO /BREPRO /DEFAULTLIB:KERNEL32.LIB /DEFAULTLIB:USER32.LIB /DLL /DYNAMICBASE /ENTRY:_DllMainCRTStartup /EXPORT:PrintUIEntryW /EXPORT:UsersRunDllW /LARGEADDRESSAWARE /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NOLOGO /NXCOMPAT /OPT:ICF,REF /OUT:$@ /RELEASE /STUB:UACAMOLE.DOS /SUBSYSTEM:WINDOWS /SWAPRUN:CD,NET /VERSION:0.815 UACAMOLE.OBJ

UACAMOLE.DOS: UACAMOLE.MAK
	CERTUTIL.EXE /DecodeHex /F /V << $@
4D 5A 90 00 01 00 00 00 04 00 00 00 FF FF 00 00   MZ..............
E0 00 00 00 43 00 00 00 40 00 00 00 00 00 00 00   ....C...@.......
00 00 00 00 19 57 04 27 00 00 00 00 00 00 00 00   .....W.'........
00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00   ................
28 43 29 6F 70 79 72 69 67 68 74 20 32 30 30 34   (C)opyright 2004
2D 32 30 32 33 2C 20 53 74 65 66 61 6E 20 4B 61   -2023, Stefan Ka
6E 74 68 61 6B 20 3C 73 74 65 66 61 6E 2E 6B 61   nthak <stefan.ka
6E 74 68 61 6B 40 6E 65 78 67 6F 2E 64 65 3E 0D   nthak@nexgo.de>.
0A 07 24 0E 1F 33 D2 B4 09 CD 21 B8 01 4C CD 21   .....3....!..L.!
<<NOKEEP

UACAMOLE.OMF: UACAMOLE.MAK
	ML.EXE /Brepro /Fo$@ /nologo /omf /Ta<< /W3 /X
; Copyright (C) 2004-2023, Stefan Kanthak <stefan.kanthak@nexgo.de>

	.model	tiny

e_header struct				; DOS .EXE header
e_magic	word	5A4Dh			; magic number "MZ"
e_cblp	word	stop - header		; bytes on last page of file
e_cp	word	1			; pages in file
e_crlc	word	0			; number of relocation table entries
e_cparhdr word	sizeof header / 16	; size of header in paragraphs
e_minalloc word	0			; minimum extra paragraphs needed
e_maxalloc word	65535			; maximum extra paragraphs needed
e_ss	word	0			; initial (PSP relative) SS value
e_sp	word	stack - header		; initial (SS relative) SP value
e_csum	word	0			; checksum
e_ip	word	sizeof message		; initial (CS relative) IP value
e_cs	word	0			; initial (PSP relative) CS value
e_lfarlc word	sizeof header		; file address of relocation table
e_ovno	word	0			; overlay number
e_res	word	4 dup (0)		; reserved words
e_oemid	word	5719h			; OEM identifier
e_oeminfo word	2704h			; OEM information
e_res2	word	10 dup (0)		; reserved words
e_lfanew dword	0			; file address of new exe header
e_header ends

	.code

header	e_header <>

message	byte	"(C)opyright 2004-2023, Stefan Kanthak <stefan.kanthak@nexgo.de>", 0Dh, 0Ah, 07h, 24h

start:
	push	cs
	pop	ds
	xor	dx, dx
	mov	ah, 09h
	int	21h

	mov	ax, 4C01h
	int	21h
stop:
	dword	20 dup (?)
stack:
	end	start
<<NOKEEP

UACAMOLE.OBJ: UACAMOLE.MAK
	CL.EXE /Brepro /c /Fo$@ /GA /GF /Gw /Gy /nologo /O1 /Oi /Tc<< /W4 /wd4100 /we4013 /Zl
// Copyright (C) 2004-2023, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/7f0aews7.aspx

#pragma comment(compiler)

#define STRICT
#define UNICODE
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

__declspec(dllimport)
INT	WINAPI	MessageBoxTimeoutA(HWND   hwnd,
		                   LPCSTR lpText,
		                   LPCSTR lpCaption,
		                   UINT   uType,
		                   WORD   wLanguageId,
		                   DWORD  dwMilliseconds);

__declspec(dllimport)
INT	WINAPI	MessageBoxTimeoutW(HWND    hwnd,
		                   LPCWSTR lpText,
		                   LPCWSTR lpCaption,
		                   UINT    uType,
		                   WORD    wLanguageId,
		                   DWORD   dwMilliseconds);

#ifdef UNICODE
#define MessageBoxTimeout	MessageBoxTimeoutW
#else
#define MessageBoxTimeout	MessageBoxTimeoutA
#endif

// https://msdn.microsoft.com/en-us/library/dswaw1wk.aspx
// https://msdn.microsoft.com/en-us/library/adf7db3x.aspx

#define wmemcpy	__movsw

// https://technet.microsoft.com/en-us/library/ee624057.aspx

__declspec(dllexport)
VOID	WINAPI	PrintUIEntryW(HWND      hWindow,
		              HINSTANCE hInstance,
		              LPCWSTR   lpszCmdLine,
		              INT       nCmdShow)
{
	// https://msdn.microsoft.com/en-us/library/ms683156.aspx
	// https://msdn.microsoft.com/en-us/library/dd373908.aspx

	if (MessageBoxTimeout(hWindow,
#if 0
	                      lpszCmdLine,
#else
	                      GetCommandLine(),
#endif
	                      __LPREFIX(__FUNCTION__) L"() entry point",
	                      MB_OK,
	                      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
	                      3210UL) == 0)
		/* no error handling */;
}

// https://support.microsoft.com/en-us/kb/164787

__declspec(dllexport)
VOID	WINAPI	UsersRunDllW(HWND      hWindow,
		             HINSTANCE hInstance,
		             LPCWSTR   lpszCmdLine,
		             INT       nCmdShow)
{
	// https://msdn.microsoft.com/en-us/library/ms683156.aspx
	// https://msdn.microsoft.com/en-us/library/dd373908.aspx

	if (MessageBoxTimeout(hWindow,
#if 0
	                      lpszCmdLine,
#else
	                      GetCommandLine(),
#endif
	                      __LPREFIX(__FUNCTION__) L"() entry point",
	                      MB_OK,
	                      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
	                      3210UL) == 0)
		/* no error handling */;
}

// https://msdn.microsoft.com/en-us/library/3hxxtd06.aspx
// https://msdn.microsoft.com/en-us/library/988ye33t.aspx
// https://msdn.microsoft.com/en-us/library/ms682583.aspx
// https://msdn.microsoft.com/en-us/library/ms682596.aspx

__declspec(safebuffers)
BOOL	WINAPI	_DllMainCRTStartup(HMODULE hModule,
		                   DWORD   dwReason,
		                   LPVOID  lpReserved)
{
	HANDLE	hWindows;
	WCHAR	szWindows[MAX_PATH];
	DWORD	dwWindows;
	WCHAR	szModule[MAX_PATH];
	DWORD	dwModule;
	WCHAR	szProcess[MAX_PATH];
	DWORD	dwProcess;
	CHAR	szOutput[1024];
	DWORD	dwOutput;

	if (dwReason != DLL_PROCESS_ATTACH)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/ms724403.aspx

	if (GetSystemWindowsDirectory(szWindows,
	                              sizeof(szWindows) / sizeof(*szWindows)) == 0UL)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/aa364991.aspx

	if (GetTempFileName(szWindows, L"UAC", 0U, szWindows) == 0U)
		return TRUE;

	// https://msdn.microsoft.com/en-us/library/aa363858.aspx

	hWindows = CreateFile(szWindows,
	                      FILE_APPEND_DATA | SYNCHRONIZE,
	                      FILE_SHARE_READ | FILE_SHARE_WRITE,
	                      (LPSECURITY_ATTRIBUTES) NULL,
	                      OPEN_EXISTING,
	                      FILE_FLAG_WRITE_THROUGH,
	                      (HANDLE) NULL);

	if (hWindows == INVALID_HANDLE_VALUE)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/ms683197.aspx

	dwModule = GetModuleFileName(hModule,
	                             szModule,
	                             sizeof(szModule) / sizeof(*szModule));

	if (dwModule != 0UL)
		szModule[dwModule] = L'\0';
	else
		// https://msdn.microsoft.com/en-us/library/dswaw1wk.aspx

		wmemcpy(szModule, L"<unknown>", sizeof("<unknown>"));

	// https://msdn.microsoft.com/en-us/library/ms683197.aspx

	dwProcess = GetModuleFileName((HMODULE) NULL,
	                              szProcess,
	                              sizeof(szProcess) / sizeof(*szProcess));

	if (dwProcess != 0UL)
		szProcess[dwProcess] = L'\0';
	else
		// https://msdn.microsoft.com/en-us/library/dswaw1wk.aspx

		wmemcpy(szProcess, L"<unknown>", sizeof("<unknown>"));

	// https://msdn.microsoft.com/en-us/library/ms647550.aspx

	dwOutput = wsprintfA(szOutput,
	                     "Module \'%ls\' loaded at address 0x%p in process \'%ls\'\r\n",
	                     szModule, hModule, szProcess);

	// https://msdn.microsoft.com/en-us/library/aa365747.aspx

	if (!WriteFile(hWindows,
	               szOutput,
	               dwOutput * sizeof(*szOutput),
	               &dwWindows,
	               (LPOVERLAPPED) NULL))
		/* no error handling */;
	else
		if (dwWindows != dwOutput * sizeof(*szOutput))
			/* no error handling */;

	// https://msdn.microsoft.com/en-us/library/ms724211.aspx

	if (!CloseHandle(hWindows))
		/* no error handling */;

	return TRUE;
}
<<NOKEEP
